# This file was auto-generated by Fern from our API Definition.

import typing
from ...types.child_tool_rule import ChildToolRule
from ...types.init_tool_rule import InitToolRule
from ...types.terminal_tool_rule import TerminalToolRule
from ...types.conditional_tool_rule import ConditionalToolRule

CreateAgentRequestToolRulesItem = typing.Union[ChildToolRule, InitToolRule, TerminalToolRule, ConditionalToolRule]
