# This file was auto-generated by Fern from our API Definition.

import typing
from .function_call_input import FunctionCallInput

ChatCompletionRequestFunctionCall = typing.Union[typing.Literal["none"], typing.Literal["auto"], FunctionCallInput]
