# This file was auto-generated by Fern from our API Definition.

import typing

ToolType = typing.Union[
    typing.Literal["custom", "letta_core", "letta_memory_core", "letta_multi_agent_core"], typing.Any
]
