# This file was auto-generated by Fern from our API Definition.

import typing
from ....types.letta_schemas_message_message import LettaSchemasMessageMessage
from ....types.letta_message_union import LettaMessageUnion

MessagesListResponse = typing.Union[typing.List[LettaSchemasMessageMessage], typing.List[LettaMessageUnion]]
